/*!
 * @file signal_connector.h
 * @brief Definition of a SignalConnector class
*/

#ifndef _SIGNAL_CONNECTOR_H
#define _SIGNAL_CONNECTOR_H

#include "executor/executor.h"

namespace OrxGui
{

#define DeclareSignal(name, params) \
  public: \
   void signal_ ##connect ##name(const SignalConnector& connector) { \
     name ## connected.push_back(connector); \
   }\
  private: \
   void  signal_ ## name params { \
     for (unsigned int i = 0; i < name ## connected . size(); i++) \
       name ## connected[i] ("TEST"); \
   }\
   std::vector<SignalConnector> name ## connected

  //! A class for Conncting Signals to Objects, inside of the GUI
  class SignalConnector
  {
  public:
    SignalConnector();
    SignalConnector(BaseObject* object, const Executor* exec);
    SignalConnector(const SignalConnector& signalConnector);
    ~SignalConnector();

    SignalConnector& operator=(const SignalConnector& signalConnector);

    void operator()(const std::string& parameters) const;
    void execute(const std::string& parameters) const { (*this)(parameters); };

    bool isClean() const { return (this->object == NULL || this->exec == NULL); }


  private:
    BaseObject*          object;         //!< The object to call.
    const Executor*      exec;           //!< The Executor, that will be called, on object.
  };
}
#endif /* _SIGNAL_CONNECTOR_H */
